<?php

namespace Drupal\subman\Plugin\Block;

use Drupal\subman\SubmanWebhookException;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\subman\SubmanSync;
use Drupal\subman\SubmanUtilities;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a 'SignupBlock' block.
 *
 * @Block(
 *  id = "subman.signup_block",
 *  admin_label = @Translation("Signup block"),
 * )
 */
class SignupBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Drupal\subman\SubmanSync definition.
   *
   * @var \Drupal\subman\SubmanSync
   */
  protected $submanSync;

  /**
   * Drupal\subman\SubmanUtilities definition.
   *
   * @var \Drupal\subman\SubmanUtilities
   */
  protected $utils;

  /**
   * Symfony\Component\HttpFoundation\RequestStack definition.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a new SignupBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    SubmanSync $subman_sync,
    SubmanUtilities $utils,
    RequestStack $request_stack
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->submanSync = $subman_sync;
    $this->requestStack = $request_stack;
    $this->utils = $utils;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('subman.sync'),
      $container->get('subman.utilities'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [
      '#cache' => ['max-age' => 0],
    ];
    $id = $this->requestStack->getCurrentRequest()->get('id');

    try {
      if (!empty($id)) {
        $pre_registration_form = $this->utils->config('pre_registration_form', 'Drupal\subman\Form\PreRegistrationForm');
        if (!empty($pre_registration_form)) {
          $build['pre_registration_form'] = \Drupal::formBuilder()->getForm($pre_registration_form, $id);
        }
        else {
          $build['embed'] = $this->submanSync->buildEmbedSignup($id);
        }
      }
      else {
        $build['embed']['#markup'] = $this->t('No id specified. Time: ' . time());
      }
    }
    catch (SubmanWebhookException $e) {
      \Drupal::logger('subman')->error('Exception catched to SignupBlock::build(): ' . $e->getMessage());
      $build['#markup'] = $this->t('An error occured.');
    }

    return $build;
  }

}
